var ec2ui_AMITreeView = {
    COLNAMES: ['ami.id','ami.location','ami.state','ami.owner','ami.isPublic'],
    launchPermissionList : new Array(),
    imageIdRegex : new RegExp("^ami-"),

    launchNewInstances : function () {
        var image = this.getSelectedImage();
        if (image == null) return;
        var retVal = {ok:null};
        window.openDialog("chrome://ec2ui/content/dialog_new_instances.xul", null, "chrome,centerscreen,modal", image, ec2ui_session, retVal);
        
        if (retVal.ok) {
            var me = this;
            var wrap = function(list) {
                if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                    ec2ui_InstancesTreeView.refresh();
                    ec2ui_InstancesTreeView.selectByInstanceIds(list);
                }
            }
            ec2ui_session.controller.runInstances(
               retVal.imageId, 
               retVal.kernelId, 
               retVal.ramdiskId, 
               retVal.minCount, 
               retVal.maxCount, 
               retVal.keyName, 
               retVal.securityGroups, 
               retVal.userData, 
               retVal.properties, 
               retVal.instanceType,
               retVal.placement,
               wrap);
        }
    },
    
    registerNewImage : function () {
        var retVal = {ok:null,manifestPath:null};
        window.openDialog("chrome://ec2ui/content/dialog_register_image.xul", null, "chrome,centerscreen,modal", ec2ui_session, retVal);
        
        if (retVal.ok) {
            var me = this;
            var wrap = function(x) { 
                if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                    me.refresh(); 
                    me.selectByImageId(x);
                }
            }
            ec2ui_session.controller.registerImage(retVal.manifestPath, wrap);
        }
    },
    
    deregisterImage  : function () {
        var index =  this.selection.currentIndex;
        if (index == -1) return;
        
        var image = this.imageList[this.selection.currentIndex];
        var confirmed = confirm("Deregister AMI "+image.id+" ("+image.location+")?");
        if (!confirmed)
            return;
       
        var me = this;
        var wrap = function() { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refresh(); 
            }
        }
        ec2ui_session.controller.deregisterImage(image.id, wrap);
    },

    getLaunchPermissionsList : function() {
        return document.getElementById("ec2ui.launchpermissions.list");
    },
    
    getSelectedLaunchPermission : function() {
        var item = this.getLaunchPermissionsList().getSelectedItem(0);
        if (item == null) return null;
        return item.value;
    },
    
    selectLaunchPermissionByName    : function(name) {
        var list = this.getLaunchPermissionsList();
        for(var i in this.launchPermissionList) {
            if (this.launchPermissionList[i] == name) {
                list.selectedIndex = i;
                return;
            }
        }
        
        // In case we don't find a match (which is probably a bug).
        list.selectedIndex = 0;
    },
    
    refreshLaunchPermissions : function() {
        var image = this.getSelectedImage();
        if (image == null) return;
        if (image.state == "deregistered") return;
        
        var me = this;
        var wrap = function(list) { me.displayLaunchPermissions(list); }
        ec2ui_session.controller.describeLaunchPermissions(image.id, wrap);
    },
    
    addGlobalLaunchPermission : function() {
        var image = this.getSelectedImage();
        if (image == null) return;
        this.addNamedPermission(image, "all");  
    },
    
    addLaunchPermission : function() {
        var image = this.getSelectedImage();
        if (image == null) return;
        var name = prompt("Please provide an EC2 user ID");
        if (name == null) return;
        this.addNamedPermission(image, name);   
    },
    
    addNamedPermission : function(image, name) {
        var me = this;
        var wrap = function() { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refreshLaunchPermissions();
                me.selectLaunchPermissionByName(name);
            }
        }
        ec2ui_session.controller.addLaunchPermission(image.id, name, wrap);
    },

    removeLaunchPermission : function() {
        var image = this.getSelectedImage();
        if (image == null) return;
        var name = this.getSelectedLaunchPermission();
        if (name == null) return;

        var confirmed = confirm("Revoke launch permissions for "+name+" on AMI "+image.id+"?");
        if (!confirmed)
            return;

        var me = this;
        var wrap = function() { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refreshLaunchPermissions(); 
            }
        }
        ec2ui_session.controller.revokeLaunchPermission(image.id, name, wrap);
    },

    resetLaunchPermissions : function() {
        var image = this.getSelectedImage();
        if (image == null) return;

        var confirmed = confirm("Reset launch permissions for AMI "+image.id+"?");
        if (!confirmed)
            return;

        var me = this;
        var wrap = function() { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refreshLaunchPermissions(); 
            }
        }
        ec2ui_session.controller.resetLaunchPermissions(image.id, wrap);
    },
    
    displayLaunchPermissions : function (permList) {
        this.launchPermissionList = permList;

        // Clear the list (is there a better way to do this?)
        var list = this.getLaunchPermissionsList();
        var count = list.getRowCount();
        for(var i = count-1; i >= 0; i--) {
            list.removeItemAt(i);
        }
        
        // Sort 'all' to the top
        permList.sort(function(x,y) {
            if (x == 'all') return -1;
            if (y == 'all') return 1;
            return x < y ? -1 : (x == y ? 0 : 1);
        });

        for(var i in permList) {
            list.appendItem(permList[i], permList[i]);
        }
    },
    
    selectionChanged : function(event) {
        BaseImagesView.selectionChanged.call(this, event);

        if (ec2ui_prefs.isAutoFetchLaunchPermissionsEnabled()) {
            this.refreshLaunchPermissions();
        }
    },
    
    getSearchText: function() {
        return document.getElementById('ec2ui.images.search').value;
    },

    invalidate: function() {
        this.displayLaunchPermissions([]);
        BaseImagesView.invalidate.call(this);
    },
};

// poor-man's inheritance
ec2ui_AMITreeView.__proto__ = BaseImagesView;

ec2ui_AMITreeView.register();
