var ec2ui_InstancesTreeView = {
    COLNAMES : [
       'instance.resId',
       'instance.ownerId',
       'instance.id',
       'instance.imageId',
       'instance.kernelId',
       'instance.ramdiskId',
       'instance.state',
       'instance.publicDnsName',
       'instance.privateDnsName',
       'instance.keyName',
       'instance.groups',
       'instance.reason',
       'instance.amiLaunchIdx',
       'instance.instanceType',
       'instance.launchTimeDisp',
       'instance.placement.availabilityZone',
    ],
    treeBox: null,
    selection: null,
    instanceList : new Array(),
    registered : false,

    get rowCount() { return this.instanceList.length; },

    setTree     : function(treeBox)     { this.treeBox = treeBox; },
    getCellText : function(idx, column) {
        if (idx >= this.rowCount) return "";
        return eval("this.instanceList[idx]."+column.id.slice(column.id.indexOf(".")+1));
    },
    
    getCellProperties : function(row, col, props) {
        var shortName = col.id.split(".").pop();
        if (shortName == "state") {
            var stateName = this.instanceList[row].state.replace('-','_').toLowerCase();
            var aserv = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService);
            props.AppendElement(aserv.getAtom("instance_"+stateName));
        }
    },
    
    isEditable: function(idx, column)  { return true; },
    isContainer: function(idx)         { return false;},
    isSeparator: function(idx)         { return false; },
    isSorted: function()               { return false; },
    
    getImageSrc: function(idx, column) { return ""; },
    getProgressMode : function(idx,column) {},
    getCellValue: function(idx, column) {},
    cycleHeader: function(col) {
        cycleHeader(
            col, 
            document, 
            this.COLNAMES,
            this.instanceList);
        this.treeBox.invalidate();
    },
    
    viewDetails : function(event) {
        var selected = new Array();
        for(var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                selected.push(this.instanceList[i]);
            }
        }
        if (selected.length != 1) return;
        
        window.openDialog("chrome://ec2ui/content/dialog_instance_details.xul", null, "chrome,centerscreen,modal", selected[0]);
    },

    sort : function() {
        sortView(document, this.COLNAMES, this.instanceList);
    },

    selectionChanged: function() {},
    cycleCell: function(idx, column) {},
    performAction: function(action) {},
    performActionOnCell: function(action, index, column) {},
    getRowProperties: function(idx, column, prop) {},
    getColumnProperties: function(column, element, prop) {},
    getLevel : function(idx) { return 0; },

    selectByInstanceIds : function(list) {
        this.selection.clearSelection();
        for(var i in list) {
            this.selectByInstanceId(list[i].id);
        }
    },
    
    selectByInstanceId  : function(instanceId) {
        for(var i in this.instanceList) {
            if (this.instanceList[i].id == instanceId) {
                this.selection.toggleSelect(i);
                return;
            }
        }
    },
    
    selectionChanged : function(event) {
        // When an instance is selected, select the associated AMI, ARI and AKI
        // but only if a single instance has been selected
        
        var imageId = null;
        var kernelId = null;
        var ramdiskId = null;

        for(var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                // stop early if multiple selections
                if (imageId) { return; }

                imageId = this.instanceList[i].imageId;
                kernelId = this.instanceList[i].kernelId;
                ramdiskId = this.instanceList[i].ramdiskId;
            }
        }

        if (imageId) { ec2ui_AMITreeView.selectByImageId(imageId); }
        if (kernelId) { ec2ui_KernelTreeView.selectByImageId(kernelId); }
        if (ramdiskId) { ec2ui_RamdiskTreeView.selectByImageId(ramdiskId); }
    },

    register: function() {
        if (!this.registered) {
            this.registered = true;
            ec2ui_model.registerInterest(this, 'instances');
        }
    },

    invalidate: function() {
        this.displayInstances(ec2ui_session.model.instances);
    },
    
    refresh: function() {
        ec2ui_session.controller.describeInstances();
    },

    notifyModelChanged: function(interest) {
        this.invalidate();
    },

    launchMore  : function () {
        var index =  this.selection.currentIndex;
        if (index == -1) return;
        
        var instance = this.instanceList[this.selection.currentIndex];
        var count = prompt("How many more instances of "+instance.id+"?", "1");
        if (count == null || count == "")
            return;
       
        var me = this;
        var wrap = function(list) { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                ec2ui_InstancesTreeView.refresh();
                ec2ui_InstancesTreeView.selectByInstanceIds(list);
            }
        }
        ec2ui_session.controller.runInstances(
           instance.imageId, 
           instance.kernelId, 
           instance.ramdiskId, 
           count, 
           count, 
           instance.keyName, 
           instance.groupList, 
           null, 
           null, 
           instance.instanceType, 
           instance.placement,
           wrap);
    },
    
    terminateInstance  : function () {
        var instanceIds = new Array();
        for(var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                instanceIds.push(this.instanceList[i].id);
            }
        }
        if (instanceIds.length == 0) 
            return;
        
        var confirmed = confirm("Terminate "+instanceIds.length+" instance(s)?");
        if (!confirmed)
            return;

        var me = this;
        var wrap = function() { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refresh(); 
                me.selectByInstanceIds();
            }
        }
        ec2ui_session.controller.terminateInstances(instanceIds, wrap);
    },
    
    fetchConsoleOutput  : function () {
        var instanceIds = new Array();
        for(var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                instanceIds.push(this.instanceList[i].id);
            }
        }
        if (instanceIds.length == 0) 
            return;
        if (instanceIds.length > 1) {
            alert("Please select a single instance");
            return;
        }
        
        var me = this;
        var wrap = function(instanceId, timestamp, output) { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refresh(); 
                me.showConsoleOutput(instanceId, timestamp, output);
            }
        }
        ec2ui_session.controller.getConsoleOutput(instanceIds[0], wrap);
    },
    
    showConsoleOutput : function(instanceId, timestamp, output) {
        window.openDialog("chrome://ec2ui/content/dialog_console_output.xul", null, "chrome,centerscreen,modal", instanceId, timestamp, output);
    },

    copyToClipBoard : function(fieldName) {
        var instances = new Array();
        for(var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                instances.push(this.instanceList[i]);
            }
        }
        if (instances.length == 0) 
            return;
        if (instances.length > 1) {
            alert("Please select a single instance");
            return;
        }

        copyToClipboard(eval("instances[0]."+fieldName));
    },

    sshToSelectedInstances : function() {
        for (var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                this.sshTo(this.instanceList[i].publicDnsName, this.instanceList[i].keyName);
            }
        }
    },

    sshTo : function(hostname, keyname) {
        // build arguments string, doing param substitution
        var arg_str = ec2ui_prefs.getSSHArgs();

        // ${home}
        var env = Components.classes["@mozilla.org/process/environment;1"].getService(Components.interfaces.nsIEnvironment);
        var home = null;
        if (env.exists('HOME')) {
            home = env.get("HOME");
        } else if (env.exists("HOMEDRIVE") && env.exists("HOMEPATH")) {
            home = env.get("HOMEDRIVE") + env.get("HOMEPATH");
        }
        arg_str = arg_str.replace(/\${home}/g, home || "");
       
        // ${keyname} and ${key}
        var key_template = ec2ui_prefs.getSSHKeyTemplate();
        key_template = key_template.replace(/\${keyname}/g, keyname);
        key_template = key_template.replace(/\${home}/g, home || "");
        arg_str = arg_str.replace(/\${key}/g, key_template);

        // ${user}
        arg_str = arg_str.replace(/\${user}/g, ec2ui_prefs.getSSHUser() || "");

        // ${host}
        arg_str = arg_str.replace(/\${host}/g, hostname);

        // Finally, split args into an array
        var args = tokenise(arg_str);
        var cmd = ec2ui_prefs.getSSHCommand();

        log("ssh command: "+cmd);
        log("ssh args: "+args.join(","));

        // create an nsILocalFile for the executable
        var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(cmd);
       
        // create an nsIProcess
        var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);
        try {
            process.init(file);
        } catch (e) {
            alert("Couldn't launch: " + cmd + "\n\n" + e.message);
            return;
        }

        try {
            // Run the process.
            // If first param is true, calling thread will be blocked until
            // called process terminates.
            // Second and third params are used to pass command-line arguments
            process.run(false, args, args.length);
        } catch (e) {
            alert("Couldn't launch: " + cmd + "\nWith arguments: " + args.join(" ") + "\n\n" + e.message);
        }
    },

    rebootInstance  : function () {
        var instanceIds = new Array();
        for(var i in this.instanceList) {
            if (this.selection.isSelected(i)) {
                instanceIds.push(this.instanceList[i].id);
            }
        }
        if (instanceIds.length == 0) 
            return;
        
        var confirmed = confirm("Reboot "+instanceIds.length+" instance(s)?");
        if (!confirmed)
            return;

        var me = this;
        var wrap = function(list) { 
            if (ec2ui_prefs.isRefreshOnChangeEnabled()) { 
                me.refresh(); 
                me.selectByInstanceIds(list);
            }
        }
        ec2ui_session.controller.rebootInstances(instanceIds, wrap);
    },
    
    displayInstances : function (instanceList) {
        this.treeBox.rowCountChanged(0, -this.instanceList.length);
        this.instanceList = instanceList;
        this.treeBox.rowCountChanged(0, this.instanceList.length);
        this.sort();
        this.selection.clearSelection();
    }
};

ec2ui_InstancesTreeView.register();
