// "Classes" representing objects like AMIs, Instances etc.
function Credential(name, accessKey, secretKey) {
    this.name = name;
    this.accessKey = accessKey;
    this.secretKey = secretKey;
}

function AccountIdName(id, name) {
    this.accountid = id;
    this.displayname = name;
}

function AMI(imageId, location, state, owner, isPublic) {
    this.id = imageId;
    this.location = location;
    this.state = state;
    this.owner = owner;
    this.isPublic = isPublic;
}

function Instance(resId, ownerId, groupList, instanceId, imageId, kernelId, ramdiskId, state, publicDnsName, privateDnsName, keyName, reason, amiLaunchIdx, instanceType, launchTime, placement) {
    this.resId = resId;
    this.ownerId = ownerId;
    this.groupList = groupList;
    this.id = instanceId;
    this.imageId = imageId;
    this.kernelId = kernelId;
    this.ramdiskId = ramdiskId;
    this.state = state;
    this.publicDnsName = publicDnsName;
    this.privateDnsName = privateDnsName;
    this.keyName = keyName;
    this.reason = reason;
    this.amiLaunchIdx = amiLaunchIdx;
    this.instanceType = instanceType;
    this.launchTime = launchTime;
    this.launchTimeDisp = launchTime.strftime('%Y-%m-%d %H:%M:%S');

    this.groups = this.groupList.join(', ');

    this.placement = placement;
}

function KeyPair(name, fingerprint) {
    this.name = name;
    this.fingerprint = fingerprint;
}

function SecurityGroup(ownerId, name, description, permissions) {
    this.ownerId = ownerId;
    this.name = name;
    this.description = description;
    this.permissions = permissions;
}

function Permission(protocol, fromPort, toPort, groupTuples, ipRanges) {
    this.protocol = protocol;
    this.fromPort = fromPort;
    this.toPort = toPort;
    this.groupTuples = groupTuples; // userId+groupName tuples
    this.ipRanges = ipRanges;           // CIDRs
    
    var tuples = new Array();
    for(var i in this.groupTuples) {
        tuples.push(this.groupTuples[i].join(':'));
    }
    this.groups = tuples.join(', ');
    this.cidrs = this.ipRanges.join(', ');
}

function AvailabilityZone(name, state) {
    this.name = name;
    this.state = state;
}

function AddressMapping(address, instanceid) {
    this.address = address;
    this.instanceid = instanceid;
}

// Global model: home to things like lists of data that need to be shared (known AMIs, keypairs etc)
var ec2ui_model = {
    components      : new Array(),
    componentInterests : new Object(),

    images          : new Array(),
    instances       : new Array(),
    keypairs        : null,
    azones          : null,
    securityGroups  : null,
    addresses       : null,
    
    invalidate : function() {
        // reset all lists, these will notify their associated views
        this.updateImages([]);
        this.updateInstances([]);
        this.updateKeypairs(null);
        this.updateSecurityGroups(null);
        this.updateAvailabilityZones(null);
        this.updateAddresses(null);
    },

    notifyComponents: function(interest) {
        var comps = this.componentInterests[interest] || [];
        for (var i in comps) {
            comps[i].notifyModelChanged(interest);
        }
    },  
    
    registerInterest : function(component, interest) {
        if (!this.componentInterests[interest]) {
            this.componentInterests[interest] = [];
        }
        this.componentInterests[interest].push(component);
    },
    
    updateImages : function(list) {
        this.images = list;
        this.notifyComponents("images");
    },
    
    updateInstances : function(list) {
        this.instances = list;
        this.notifyComponents("instances");
    },
    
    updateKeypairs : function(list) {
        this.keypairs = list;
        this.notifyComponents("keypairs");
    },

    getKeypairs : function() {
        if (this.keypairs == null) {
            ec2ui_session.controller.describeKeypairs();
        }
        return this.keypairs;
    },

    updateSecurityGroups : function(list) {
        this.securityGroups = list;
        this.notifyComponents("securitygroups");
    },

    getSecurityGroups : function() {
        if (this.securityGroups == null) {
            ec2ui_session.controller.describeSecurityGroups();
        }
        return this.securityGroups;
    },

    updateAddresses: function(list) {
        this.addresses = list;
        this.notifyComponents("addresses");
    },

    updateAvailabilityZones : function(list) {
        this.azones = list;
        this.notifyComponents("azones");
    },

    getAvailabilityZones : function() {
        if (this.azones == null) {
            ec2ui_session.controller.describeAvailabilityZones();
        }
        return this.azones;
    }
}
